/// <reference types="cheerio" />
import { IQuery } from '../common/IQuery';
import { IBuildConfig, ISettings } from '../common/IConfig';
import { IDriver } from '../common/IDriver';
import { Deferred } from '../types/Deferred';
export declare class CherrioQuery extends IQuery<CheerioElement> {
    protected _onFn(node: CheerioElement, type: string, cb: Function): Promise<any>;
    protected _onOnceFn(node: CheerioElement, type: string, cb: Function): Promise<any>;
    protected _offFn(node: CheerioElement, type: string, cb: Function): Promise<any>;
    hasClassFn(node: CheerioElement, name: string): Deferred<boolean>;
    addClassFn(node: CheerioElement, name: string): Deferred<void>;
    removeClassFn(node: CheerioElement, name: string): Deferred<void>;
    toggleClassFn(node: CheerioElement, name: string): Deferred<void>;
    textGetFn(node: CheerioElement): Deferred<string>;
    textSetFn(node: CheerioElement, text: string): Deferred<void>;
    htmlOuterGetFn(node: CheerioElement): Deferred<string>;
    htmlGetFn(node: CheerioElement): Deferred<string>;
    htmlSetFn(node: CheerioElement, text: string): Deferred<void>;
    appendFn(node: CheerioElement, html: string): Deferred<void>;
    prependFn(node: CheerioElement, html: string): Deferred<void>;
    beforeFn(node: CheerioElement, html: string): Deferred<void>;
    afterFn(node: CheerioElement, html: string): Deferred<void>;
    cssGet(node: CheerioElement, prop: string): Promise<any>;
    cssSet(node: CheerioElement, css: {
        [key: string]: any;
    }): Deferred<void>;
    heightGetFn(node: CheerioElement): Promise<number>;
    widthGetFn(node: CheerioElement): Promise<number>;
    innerHeightFn(node: CheerioElement): Promise<number>;
    innerWidthFn(node: CheerioElement): Promise<number>;
    getBoundingClientRect(node: CheerioElement): Promise<{
        top: number;
        left: number;
        width: number;
        height: number;
    }>;
    getPosition(node: CheerioElement): Promise<{
        top: number;
        left: number;
    }>;
    scrollTopGetFn(node: CheerioElement): Promise<number>;
    scrollTopSetFn(node: CheerioElement, scroll: number): Deferred<void>;
    scrollLeftGetFn(node: CheerioElement): Promise<number>;
    scrollLeftSetFn(node: CheerioElement, scroll: number): Deferred<void>;
    evalFn(node: CheerioElement, mix: Function | string, ...args: any[]): Promise<any>;
    clickFn(node: CheerioElement): Promise<void>;
    triggerFn(node: CheerioElement, type: string, ...args: any[]): Promise<void>;
    selectFn(node: CheerioElement, ...args: any[]): Promise<any>;
    focusFn(node: CheerioElement): Promise<void>;
    blurFn(node: CheerioElement): Promise<void>;
    sendKeysFn(node: CheerioElement, mix: any): Promise<void>;
    typeFn(node: CheerioElement, str: string): Promise<void>;
    pressFn(node: CheerioElement, str: string): Promise<void>;
    removeFn(node: CheerioElement): Promise<void>;
    attrGetFn(node: CheerioElement, prop: string): Promise<any>;
    attrSetFn(node: CheerioElement, attr: {
        [key: string]: any;
    }): Deferred<void>;
    valGetFn(node: CheerioElement): Promise<any>;
    valSetFn(node: CheerioElement, value: any): Deferred<void>;
    dataGetFn(node: CheerioElement, key: string): Promise<any>;
    dataSetFn(node: CheerioElement, data: object): Deferred<void>;
    protected propGetFn(node: CheerioElement, key: string): Promise<any>;
    protected propSetFn(node: CheerioElement, data: object): Deferred<void>;
    findFn(node: CheerioElement, selector: string): Deferred<CheerioElement[]>;
    matchesFn(node: CheerioElement, selector: string): Deferred<boolean>;
    parentFn(node: CheerioElement): Promise<CheerioElement>;
    closestFn(node: CheerioElement, sel: string): Promise<CheerioElement>;
    childrenFn(node: CheerioElement, sel?: string): Promise<CheerioElement[]>;
    nextFn(node: CheerioElement, sel?: string): Promise<CheerioElement>;
    protected getField<T>(node: CheerioElement, field: string): Deferred<T>;
    protected setField(node: CheerioElement, obj: any): Deferred<void>;
    protected setField(node: CheerioElement, field: string, val: any): Deferred<void>;
    protected callField<T>(node: CheerioElement, field: string, ...args: any[]): Deferred<T>;
    static newAsync(mix?: any, parent?: IQuery<CheerioElement>): CherrioQuery;
    unlock(): void;
    static build(config: IBuildConfig, setts?: ISettings): Promise<IDriver>;
    static load(url: string, config: IBuildConfig, setts?: ISettings): CherrioQuery;
    static fetch(url: string, config: IBuildConfig, setts?: ISettings): CherrioQuery;
    static setDriver(driver: IDriver): void;
    static getDriver(config: IBuildConfig, setts?: ISettings): Promise<IDriver>;
    static unlockDriver(mix: any): void;
}
