"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const IQuery_1 = require("../common/IQuery");
const dfr_1 = require("../utils/dfr");
const NetworkDriver_1 = require("../fetch/NetworkDriver");
const CheerioUtils_1 = require("./CheerioUtils");
const $ = require("cheerio");
class CherrioQuery extends IQuery_1.IQuery {
    _onFn(node, type, cb) {
        throw new Error('Method not implemented.');
    }
    _onOnceFn(node, type, cb) {
        throw new Error('Method not implemented.');
    }
    _offFn(node, type, cb) {
        throw new Error('Method not implemented.');
    }
    hasClassFn(node, name) {
        return dfr_1.dfr_resolve($(node).hasClass(name));
    }
    addClassFn(node, name) {
        $(node).addClass(name);
        return dfr_1.dfr_resolve();
    }
    removeClassFn(node, name) {
        $(node).removeClass(name);
        return dfr_1.dfr_resolve();
    }
    toggleClassFn(node, name) {
        $(node).toggleClass(name);
        return dfr_1.dfr_resolve();
    }
    textGetFn(node) {
        // Cheerio returns empty string on `text` for script elements
        const method = node.tagName === 'script' ? 'html' : 'text';
        return dfr_1.dfr_resolve($(node)[method]());
    }
    textSetFn(node, text) {
        $(node).text(text);
        return dfr_1.dfr_resolve();
    }
    htmlOuterGetFn(node) {
        return dfr_1.dfr_resolve($.html(node));
    }
    htmlGetFn(node) {
        return dfr_1.dfr_resolve(CheerioUtils_1.CheerioUtils.fromNode(node).html());
    }
    htmlSetFn(node, text) {
        $(node).html(text);
        return dfr_1.dfr_resolve();
    }
    appendFn(node, html) {
        $(node).append(html);
        return dfr_1.dfr_resolve();
    }
    prependFn(node, html) {
        $(node).prepend(html);
        return dfr_1.dfr_resolve();
    }
    beforeFn(node, html) {
        $(node).insertBefore(html);
        return dfr_1.dfr_resolve();
    }
    afterFn(node, html) {
        $(node).insertAfter(html);
        return dfr_1.dfr_resolve();
    }
    cssGet(node, prop) {
        return dfr_1.dfr_resolve($(node).css(prop));
    }
    cssSet(node, css) {
        $(node).css(css);
        return dfr_1.dfr_resolve();
    }
    heightGetFn(node) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Cheerio driver does not support dimensions feature');
        });
    }
    widthGetFn(node) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Cheerio driver does not support dimensions feature');
        });
    }
    innerHeightFn(node) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    innerWidthFn(node) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    getBoundingClientRect(node) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    getPosition(node) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Cheerio driver does not support dimensions feature');
        });
    }
    scrollTopGetFn(node) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    scrollTopSetFn(node, scroll) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    scrollLeftGetFn(node) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    scrollLeftSetFn(node, scroll) {
        throw new Error('Cheerio driver does not support dimensions feature');
    }
    evalFn(node, mix, ...args) {
        throw new Error('Eval is not supported with Cheerio Driver');
    }
    //#region Events
    clickFn(node) {
        throw new Error('Cheerio driver does not support manipulation feature');
    }
    triggerFn(node, type, ...args) {
        throw new Error('Cheerio driver does not support manipulation feature');
    }
    selectFn(node, ...args) {
        throw new Error('Cheerio driver does not support manipulation feature');
    }
    focusFn(node) {
        throw new Error('FOCUS is not supported in Cheerio');
    }
    blurFn(node) {
        throw new Error('BLUR is not supported in Cheerio');
    }
    sendKeysFn(node, mix) {
        throw new Error('SEND_KEYS is not supported in Cheerio');
    }
    typeFn(node, str) {
        throw new Error('TYPE is not supported in Cheerio');
    }
    pressFn(node, str) {
        throw new Error('PRESS is not supported in Cheerio');
    }
    //#endregion
    //#region Manipulate
    removeFn(node) {
        $(node).remove();
        return dfr_1.dfr_resolve();
    }
    //#endregion
    //#region Properties
    attrGetFn(node, prop) {
        return dfr_1.dfr_resolve($(node).attr(prop));
    }
    attrSetFn(node, attr) {
        for (let key in attr) {
            $(node).attr(key, attr[key]);
        }
        return dfr_1.dfr_resolve();
    }
    valGetFn(node) {
        return dfr_1.dfr_resolve($(node).val());
    }
    valSetFn(node, value) {
        $(node).val(value);
        return dfr_1.dfr_resolve();
    }
    dataGetFn(node, key) {
        return dfr_1.dfr_resolve($(node).data(key));
    }
    dataSetFn(node, data) {
        for (let key in data) {
            $(node).data(key, data[key]);
        }
        return dfr_1.dfr_resolve();
    }
    propGetFn(node, key) {
        return dfr_1.dfr_resolve($(node).prop(key));
    }
    propSetFn(node, data) {
        for (let key in data) {
            $(node).prop(key, data[key]);
        }
        return dfr_1.dfr_resolve();
    }
    //#endregion
    findFn(node, selector) {
        let arr = $(node).find(selector).toArray();
        return dfr_1.dfr_resolve(arr);
    }
    matchesFn(node, selector) {
        return dfr_1.dfr_resolve($(node).is(selector));
    }
    parentFn(node) {
        let el = $(node).parent().get(0);
        return dfr_1.dfr_resolve(el);
    }
    closestFn(node, sel) {
        let el = $(node).closest(sel).get(0);
        return dfr_1.dfr_resolve(el);
    }
    childrenFn(node, sel) {
        let arr = $(node).children(sel).toArray();
        return dfr_1.dfr_resolve(arr);
    }
    nextFn(node, sel) {
        let next = $(node).next(sel).get(0);
        return dfr_1.dfr_resolve(next);
    }
    getField(node, field) {
        return node[field];
    }
    setField(node, mix, val) {
        if (arguments.length === 2) {
            for (let key in mix) {
                node[key] = mix[key];
            }
            return dfr_1.dfr_resolve();
        }
        node[mix] = val;
        return dfr_1.dfr_resolve();
    }
    callField(node, field, ...args) {
        return dfr_1.dfr_resolve(node[field](...args));
    }
    static newAsync(mix, parent) {
        let query = new CherrioQuery(mix);
        query.ctx.owner = parent;
        query.then = query.ctx.thener;
        return query;
    }
    //#region driver utils
    unlock() {
    }
    //#endregion driver utils
    static build(config, setts) {
        throw new Error('No build for JSDom is required. Use direkt load');
    }
    static load(url, config, setts) {
        let query = new CherrioQuery();
        NetworkDriver_1.NetworkDriver.load(url, setts.opts).then(content => {
            let html = content.toString();
            let $ = CheerioUtils_1.CheerioUtils.fromHtml(html);
            query.ctx.source = html;
            query.add($);
            query.resolve(query);
        });
        return query;
    }
    static fetch(url, config, setts) {
        return this.load(url, config, setts);
    }
    static setDriver(driver) {
        throw new Error('JSDOM does not support driver');
    }
    static getDriver(config, setts) {
        throw new Error('JSDOM does not support driver');
    }
    static unlockDriver(mix) {
    }
}
exports.CherrioQuery = CherrioQuery;
function toCamelCase(property) {
    return property.replace(/\-(\w)/g, (full, char) => {
        return char.toUpperCase();
    });
}
