"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async_1 = require("../utils/async");
const atma_utils_1 = require("atma-utils");
const dfr_1 = require("../utils/dfr");
const arr_1 = require("../utils/arr");
const SelectorsEx_1 = require("./SelectorsEx");
class IQueryCtx {
    newSync(arr, parent) {
        let query = new this.Ctor(arr);
        query.ctx.owner = parent || this.self;
        return query;
    }
    newAsync(arr, parent) {
        let query = new this.Ctor(arr);
        query.ctx.owner = parent || this.self;
        query.then = query.ctx.thener;
        return query;
    }
}
exports.IQueryCtx = IQueryCtx;
class IQuery extends atma_utils_1.class_Dfr {
    constructor(mix) {
        super();
        this.length = 0;
        this.ctx = new IQueryCtx;
        this.ctx.self = this;
        this.ctx.thener = this.then;
        this.ctx.Ctor = this.constructor;
        this.ensureSync();
        this.add(mix);
        if (mix != null) {
            this.resolve(this);
        }
    }
    ensureSync() {
        this.then = null;
        return this;
    }
    ensureAsync() {
        if (this.then != null) {
            return this;
        }
        return this.ctx.newAsync(this, this);
    }
    resolve(...args) {
        if (args.length !== 0) {
            let x = args[0];
            if (x != null && x.then != null && x instanceof IQuery) {
                let q = new x.ctx.Ctor(x);
                args[0] = q;
            }
        }
        return super.resolve(...args);
    }
    wait(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }
    //#region CssClass
    hasClass(name) {
        return Arr.mapFirst(this, node => this.hasClassFn(node, name));
    }
    addClass(name) {
        return Arr.mutate(this, node => this.addClassFn(node, name));
    }
    removeClass(name) {
        return Arr.mutate(this, node => this.removeClassFn(node, name));
    }
    toggleClass(name) {
        return Arr.mutate(this, node => this.toggleClassFn(node, name));
    }
    //#endregion CssClass
    //#region Collection
    add(mix) {
        if (mix == null) {
            return this;
        }
        if (atma_utils_1.is_ArrayLike(mix) === true) {
            return arr_1.each(mix, this.add, this);
        }
        this[this.length++] = mix;
        return this;
    }
    eq(index) {
        return async_1.async_next(this, ($, source) => {
            if (index < source.length) {
                $.add(source[index]);
            }
        });
    }
    slice(start = 0, end) {
        return async_1.async_next(this, ($, source) => {
            var i = start;
            if (end > source.length) {
                end = source.length;
            }
            if (i < end) {
                for (; i < end; i++) {
                    $.add(source[i]);
                }
            }
        });
    }
    each(fn) {
        return async_1.async_next(this, ($, source) => {
            return async_1.async_waterfall(source, (node, i) => {
                $.add(node);
                return fn(node, i);
            });
        });
    }
    map(fn) {
        return async_1.async_map(this, fn);
    }
    toArray() {
        return dfr_1.dfr_run(resolve => {
            this.ensureAsync().done($ => {
                var arr = Array.prototype.slice.call($);
                resolve(arr);
            });
        });
    }
    text(str) {
        if (typeof str === 'undefined') {
            return async_1.async_aggr('', this, (accum, node) => {
                return this.textGetFn(node).then(val => accum + val);
            });
        }
        return async_1.async_each(this, ($, node) => {
            return this
                .textSetFn(node, str)
                .done(() => $.add(node));
        });
    }
    html(str) {
        if (typeof str === 'undefined') {
            return async_1.async_aggr('', this, (accum, node) => {
                return this.htmlGetFn(node).then(val => accum + val);
            });
        }
        return async_1.async_each(this, ($, node) => {
            return this
                .htmlSetFn(node, str)
                .done(() => $.add(node));
        });
    }
    outerHtml() {
        return async_1.async_aggr('', this, (accum, node) => {
            return this.htmlOuterGetFn(node).then(val => accum + val);
        });
    }
    append(html) {
        return Arr.mutate(this, node => {
            return this.appendFn(node, html);
        });
    }
    prepend(html) {
        return Arr.mutate(this, node => this.prependFn(node, html));
    }
    before(html) {
        return Arr.mutate(this, node => this.beforeFn(node, html));
    }
    after(html) {
        return Arr.mutate(this, node => this.afterFn(node, html));
    }
    css(mix, val) {
        if (arguments.length === 1 && typeof mix === 'string') {
            return Arr.mapFirst(this, node => this.cssGet(node, mix));
        }
        let css = arguments.length === 1 ? mix : { [mix]: val };
        return Arr.mutate(this, node => this.cssSet(node, css));
    }
    height(val) {
        if (val == null) {
            return Arr.mapFirst(this, node => this.heightGetFn(node));
        }
        return this.css('height', val);
    }
    innerHeight() {
        return Arr.mapFirst(this, node => this.innerHeightFn(node));
    }
    width(val = null) {
        if (val == null) {
            return Arr.mapFirst(this, node => this.widthGetFn(node));
        }
        return this.css('width', val);
    }
    innerWidth() {
        return Arr.mapFirst(this, node => this.innerWidthFn(node));
    }
    offset() {
        return Arr.mapFirst(this, node => this.getBoundingClientRect(node));
    }
    position() {
        return Arr.mapFirst(this, node => this.getPosition(node));
    }
    scrollTop(mix) {
        if (arguments.length === 0) {
            return Arr.mapFirst(this, node => this.scrollTopGetFn(node));
        }
        return Arr.mutate(this, node => this.scrollTopSetFn(node, mix));
    }
    scrollLeft(mix) {
        if (arguments.length === 0) {
            return Arr.mapFirst(this, node => this.scrollLeftGetFn(node));
        }
        return Arr.mutate(this, node => this.scrollLeftSetFn(node, mix));
    }
    //#endregion
    //#region driver
    eval(mix, ...args) {
        return Arr.mapFirst(this, node => this.evalFn(node, mix, ...args));
    }
    //#endregion
    //#region Events
    click() {
        return Arr.mutate(this, node => this.clickFn(node));
    }
    trigger(type, ...args) {
        return Arr.mutate(this, node => this.triggerFn(node, type, ...args));
    }
    select(...args) {
        return Arr.mutate(this, node => this.selectFn(node, ...args));
    }
    focus() {
        return Arr.mutate(this, node => this.focusFn(node));
    }
    blur() {
        return Arr.mutate(this, node => this.blurFn(node));
    }
    sendKeys(mix) {
        return Arr.mutate(this, node => this.sendKeysFn(node, mix));
    }
    type(str) {
        return Arr.mutate(this, node => this.typeFn(node, str));
    }
    press(str) {
        return Arr.mutate(this, node => this.pressFn(node, str));
    }
    //#endregion
    //#region Manipulate
    remove() {
        return Arr.mutate(this, node => this.removeFn(node));
    }
    attr(mix, val) {
        if (arguments.length === 1 && typeof mix === 'string') {
            return Arr.mapFirst(this, node => this.attrGetFn(node, mix));
        }
        let attr = arguments.length === 2 ? { [mix]: val } : mix;
        return Arr.mutate(this, node => this.attrSetFn(node, attr));
    }
    val(val) {
        if (arguments.length === 0) {
            return Arr.mapFirst(this, node => this.valGetFn(node));
        }
        return Arr.mutate(this, node => this.valSetFn(node, val));
    }
    data(mix, val) {
        if (arguments.length === 1 && typeof mix === 'string') {
            return Arr.mapFirst(this, node => this.dataGetFn(node, mix));
        }
        let data = arguments.length === 2 ? { [mix]: val } : mix;
        return Arr.mutate(this, node => this.dataSetFn(node, data));
    }
    prop(mix, val) {
        if (arguments.length === 1 && typeof mix === 'string') {
            return Arr.mapFirst(this, node => this.getField(node, mix));
        }
        let obj = arguments.length === 2 ? { [mix]: val } : mix;
        return Arr.mutate(this, node => this.setField(node, obj));
    }
    //#endregion
    //#region Traverse
    find(sel) {
        return SelectorsEx_1.SelectorsEx.find(this, sel, (el, sel) => {
            return async_1.async_traverse(el, (node) => this.findFn(node, sel));
        });
        //return async_traverse(this, (node: TElement) => this.findFn(node, sel));
    }
    filter(mix) {
        if (typeof mix === 'string') {
            let selector = mix;
            return async_1.async_filter(this, $single => this.matchesFn($single[0], selector));
        }
        let fn = mix;
        return async_1.async_filter(this, fn);
    }
    parent() {
        return async_1.async_traverse(this, node => {
            return this.parentFn(node);
        });
    }
    closest(sel) {
        return async_1.async_traverse(this, node => {
            return this.closestFn(node, sel);
        });
    }
    children(sel) {
        return async_1.async_traverse(this, node => {
            return this.childrenFn(node, sel);
        });
    }
    next(sel) {
        return async_1.async_traverse(this, node => {
            return this.nextFn(node, sel);
        });
    }
    on(type, cb) {
        return async_1.async_each(this, (ctx, node) => this._onFn(node, type, cb));
    }
    off(type, cb) {
        return async_1.async_each(this, (ctx, node) => this._offFn(node, type, cb));
    }
    once(type, cb = null) {
        return async_1.async_each(this, (ctx, node) => this._onOnceFn(node, type, cb));
    }
}
exports.IQuery = IQuery;
var Arr;
(function (Arr) {
    function mutate(self, mutator) {
        return async_1.async_each(self, ($, node) => {
            $.add(node);
            return mutator(node);
        });
    }
    Arr.mutate = mutate;
    function mapFirst(self, map) {
        return async_1.async_getValueOf(0, self, map);
    }
    Arr.mapFirst = mapFirst;
})(Arr || (Arr = {}));
