"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const JsdomQuery_1 = require("./JsdomQuery");
const NetworkDriver_1 = require("../fetch/NetworkDriver");
const jsdom_1 = require("jsdom");
const SelectorsEx_1 = require("../common/SelectorsEx");
exports.JsdomDriver = {
    fromHtml(html) {
        return exports.JsdomDriver.build({ html });
    },
    build(config) {
        let html = config.html;
        let jsdom = new jsdom_1.JSDOM(html);
        let el = jsdom.window.document;
        let isDocument = true;
        if (/^\s*<(?!(!?DOCTYPE|html))/i.test(html)) {
            isDocument = false;
        }
        if (config.fragment === true) {
            isDocument = false;
        }
        if (isDocument === false) {
            el = Array.from(el.body.children);
        }
        let query = new JsdomQuery_1.JsdomQuery(el);
        return query;
    },
    load(url, config) {
        let query = JsdomQuery_1.JsdomQuery.newAsync();
        NetworkDriver_1.NetworkDriver
            .load(url, config)
            .then(resp => {
            let jsdom = new jsdom_1.JSDOM(resp.body.toString());
            query.add(jsdom.window.document);
            query.resolve(query);
        }, error => {
            query.reject(error);
        });
        return query;
    },
    fetch(url, config, setts) {
        return this.load(url, config, setts);
    },
    setDriver(driver) {
        throw new Error('JSDOM does not support driver');
    },
    getDriver(config, setts) {
        throw new Error('JSDOM does not support driver');
    },
    unlockDriver(mix) {
        throw new Error('JSDOM does not support driver');
    },
    pseudo: SelectorsEx_1.SelectorsEx.pseudoFns
};
