import { IQuery } from '../common/IQuery';
import { IBuildConfig, ISettings } from '../common/IConfig';
import { IDriver } from '../common/IDriver';
import { Deferred } from '../types/Deferred';
export declare class JsdomQuery extends IQuery<Element> {
    protected _onFn(node: Element, type: string, cb: Function): Promise<any>;
    protected _onOnceFn(node: Element, type: string, cb: Function): Promise<any>;
    protected _offFn(node: Element, type: string, cb: Function): Promise<any>;
    hasClassFn(node: Element, name: string): Deferred<boolean>;
    addClassFn(node: Element, name: string): Deferred<void>;
    removeClassFn(node: Element, name: string): Deferred<void>;
    toggleClassFn(node: Element, name: string): Deferred<void>;
    textGetFn(node: Element): Deferred<string>;
    textSetFn(node: Element, text: string): Deferred<void>;
    htmlOuterGetFn(node: Element): Deferred<string>;
    htmlGetFn(node: Element): Deferred<string>;
    htmlSetFn(node: Element, text: string): Deferred<void>;
    appendFn(node: Element, html: string): Deferred<void>;
    prependFn(node: Element, html: string): Deferred<void>;
    beforeFn(node: Element, html: string): Deferred<void>;
    afterFn(node: Element, html: string): Deferred<void>;
    cssGet(node: HTMLElement, prop: string): Promise<any>;
    cssSet(node: HTMLElement, css: {
        [key: string]: any;
    }): Deferred<void>;
    heightGetFn(node: HTMLElement): Promise<number>;
    widthGetFn(node: HTMLElement): Promise<number>;
    innerHeightFn(node: Element): Promise<number>;
    innerWidthFn(node: Element): Promise<number>;
    getBoundingClientRect(node: HTMLElement): Promise<{
        top: number;
        left: number;
        width: number;
        height: number;
    }>;
    getPosition(node: Element): Promise<{
        top: number;
        left: number;
    }>;
    scrollTopGetFn(node: Element): Promise<number>;
    scrollTopSetFn(node: Element, scroll: number): Deferred<void>;
    scrollLeftGetFn(node: Element): Promise<number>;
    scrollLeftSetFn(node: Element, scroll: number): Deferred<void>;
    evalFn(node: Element, mix: Function | string, ...args: any[]): Promise<any>;
    clickFn(node: HTMLElement): Promise<void>;
    triggerFn(node: HTMLElement, type: string, ...args: any[]): Promise<void>;
    selectFn(node: Element, ...args: any[]): Promise<any>;
    focusFn(node: Element): Promise<void>;
    blurFn(node: Element): Promise<void>;
    sendKeysFn(node: Element, mix: any): Promise<void>;
    typeFn(node: Element, str: string): Promise<void>;
    pressFn(node: Element, str: string): Promise<void>;
    removeFn(node: HTMLElement): Promise<void>;
    attrGetFn(node: HTMLElement, prop: string): Promise<any>;
    attrSetFn(node: Element, attr: {
        [key: string]: any;
    }): Deferred<void>;
    valGetFn(node: Element): Promise<any>;
    valSetFn(node: Element, value: any): Deferred<void>;
    dataGetFn(node: HTMLElement, key: string): Promise<any>;
    dataSetFn(node: HTMLElement, data: object): Deferred<void>;
    protected propGetFn(node: HTMLElement, key: string): Promise<any>;
    protected propSetFn(node: Element, data: object): Deferred<void>;
    findFn(node: Element, selector: string): Deferred<Element[]>;
    matchesFn(node: Element, selector: string): Deferred<boolean>;
    parentFn(node: Element): Promise<Element>;
    closestFn(node: Element, sel: string): Promise<Element>;
    childrenFn(node: Element, sel?: string): Promise<Element[]>;
    nextFn(node: Element, sel?: string): Promise<Element>;
    protected getField<T>(node: Element, field: string): Deferred<T>;
    protected setField(node: Element, obj: any): Deferred<void>;
    protected setField(node: Element, field: string, val: any): Deferred<void>;
    protected callField<T>(node: Element, field: string, ...args: any[]): Deferred<T>;
    static newAsync(mix?: any, parent?: IQuery<Element>): JsdomQuery;
    unlock(): void;
    static build(config: IBuildConfig, setts?: ISettings): Promise<IDriver>;
    static load(url: string, config: IBuildConfig, setts?: ISettings): JsdomQuery;
    static fetch(url: string, config: IBuildConfig, setts?: ISettings): JsdomQuery;
    static setDriver(driver: IDriver): void;
    static getDriver(config: IBuildConfig, setts?: ISettings): Promise<IDriver>;
    static unlockDriver(mix: any): void;
}
