"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const IQuery_1 = require("../common/IQuery");
const jsdom_1 = require("jsdom");
const dfr_1 = require("../utils/dfr");
const NetworkDriver_1 = require("../fetch/NetworkDriver");
class JsdomQuery extends IQuery_1.IQuery {
    _onFn(node, type, cb) {
        throw new Error('Method not implemented.');
    }
    _onOnceFn(node, type, cb) {
        throw new Error('Method not implemented.');
    }
    _offFn(node, type, cb) {
        throw new Error('Method not implemented.');
    }
    hasClassFn(node, name) {
        return dfr_1.dfr_resolve(node.classList.contains(name));
    }
    addClassFn(node, name) {
        node.classList.add(name);
        return dfr_1.dfr_resolve();
    }
    removeClassFn(node, name) {
        node.classList.remove(name);
        return dfr_1.dfr_resolve();
    }
    toggleClassFn(node, name) {
        node.classList.toggle(name);
        return dfr_1.dfr_resolve();
    }
    textGetFn(node) {
        return dfr_1.dfr_resolve(node.textContent);
    }
    textSetFn(node, text) {
        node.textContent = text;
        return dfr_1.dfr_resolve();
    }
    htmlOuterGetFn(node) {
        return dfr_1.dfr_resolve(node.outerHTML);
    }
    htmlGetFn(node) {
        return dfr_1.dfr_resolve(node.innerHTML);
    }
    htmlSetFn(node, text) {
        node.innerHTML = text;
        return dfr_1.dfr_resolve();
    }
    appendFn(node, html) {
        node.insertAdjacentHTML('beforeend', html);
        return dfr_1.dfr_resolve();
    }
    prependFn(node, html) {
        node.insertAdjacentHTML('afterbegin', html);
        return dfr_1.dfr_resolve();
    }
    beforeFn(node, html) {
        node.insertAdjacentHTML('beforebegin', html);
        return dfr_1.dfr_resolve();
    }
    afterFn(node, html) {
        node.insertAdjacentHTML('afterend', html);
        return dfr_1.dfr_resolve();
    }
    cssGet(node, prop) {
        return dfr_1.dfr_resolve(node.style[toCamelCase(prop)]);
    }
    cssSet(node, css) {
        for (let key in css) {
            node.style[toCamelCase(key)] = css[key];
        }
        return dfr_1.dfr_resolve();
    }
    heightGetFn(node) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getBoundingClientRect(node)).height;
        });
    }
    widthGetFn(node) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getBoundingClientRect(node)).width;
        });
    }
    innerHeightFn(node) {
        return this.getField(node, 'offsetHeight');
    }
    innerWidthFn(node) {
        return this.getField(node, 'offsetWidth');
    }
    getBoundingClientRect(node) {
        return dfr_1.dfr_resolve(node.getBoundingClientRect());
    }
    getPosition(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let dfrTop = this.getField(node, 'offsetTop');
            let dfrLeft = this.getField(node, 'offsetLeft');
            let [top, left] = yield Promise.all([dfrTop, dfrLeft]);
            return { top, left };
        });
    }
    scrollTopGetFn(node) {
        return this.getField(node, 'scrollTop');
    }
    scrollTopSetFn(node, scroll) {
        return this.setField(node, 'scrollTop', scroll);
    }
    scrollLeftGetFn(node) {
        return this.getField(node, 'scrollLeft');
    }
    scrollLeftSetFn(node, scroll) {
        return this.setField(node, 'scrollLeft', scroll);
    }
    evalFn(node, mix, ...args) {
        throw new Error('Eval is not supported in JSDOM');
    }
    //#region Events
    clickFn(node) {
        node.click();
        return dfr_1.dfr_resolve();
    }
    triggerFn(node, type, ...args) {
        throw new Error('Trigger is not supported in JSDOM');
    }
    selectFn(node, ...args) {
        throw new Error('Select is not supported in JSDOM');
    }
    focusFn(node) {
        throw new Error('FOCUS is not supported in JSDOM');
    }
    blurFn(node) {
        throw new Error('BLUR is not supported in JSDOM');
    }
    sendKeysFn(node, mix) {
        throw new Error('SEND_KEYS is not supported in JSDOM');
    }
    typeFn(node, str) {
        throw new Error('TYPE is not supported in JSDOM');
    }
    pressFn(node, str) {
        throw new Error('PRESS is not supported in JSDOM');
    }
    //#endregion
    //#region Manipulate
    removeFn(node) {
        node.parentElement.removeChild(node);
        return dfr_1.dfr_resolve();
    }
    //#endregion
    //#region Properties
    attrGetFn(node, prop) {
        return dfr_1.dfr_resolve(node.getAttribute(prop));
    }
    attrSetFn(node, attr) {
        for (let key in attr) {
            node.setAttribute(key, attr[key]);
        }
        return dfr_1.dfr_resolve();
    }
    valGetFn(node) {
        return this.getField(node, 'value');
    }
    valSetFn(node, value) {
        return this.setField(node, 'value', value);
    }
    dataGetFn(node, key) {
        return dfr_1.dfr_resolve(node.dataset[key]);
    }
    dataSetFn(node, data) {
        for (let key in data) {
            node.dataset[key] = data[key];
        }
        return dfr_1.dfr_resolve();
    }
    propGetFn(node, key) {
        return dfr_1.dfr_resolve(node[key]);
    }
    propSetFn(node, data) {
        for (let key in data) {
            node[key] = data[key];
        }
        return dfr_1.dfr_resolve();
    }
    //#endregion
    findFn(node, selector) {
        let arr = Array.from(node.querySelectorAll(selector));
        return dfr_1.dfr_resolve(arr);
    }
    matchesFn(node, selector) {
        return dfr_1.dfr_resolve(node.matches(selector));
    }
    parentFn(node) {
        return dfr_1.dfr_resolve(node.parentElement);
    }
    closestFn(node, sel) {
        let el = node.parentElement;
        for (; el != null; el = el.parentElement) {
            el = el.parentElement;
            if (el.matches(sel)) {
                break;
            }
        }
        return dfr_1.dfr_resolve(el);
    }
    childrenFn(node, sel) {
        let arr = Array.from(node.children);
        if (sel) {
            arr = arr.filter(el => el.matches(sel));
        }
        return dfr_1.dfr_resolve(arr);
    }
    nextFn(node, sel) {
        let next = node.nextElementSibling;
        if (sel != null) {
            for (; next != null; next = next.nextElementSibling) {
                if (next.matches(sel))
                    break;
            }
        }
        return dfr_1.dfr_resolve(next);
    }
    getField(node, field) {
        return node[field];
    }
    setField(node, mix, val) {
        if (arguments.length === 2) {
            for (let key in mix) {
                node[key] = mix[key];
            }
            return dfr_1.dfr_resolve();
        }
        node[mix] = val;
        return dfr_1.dfr_resolve();
    }
    callField(node, field, ...args) {
        return dfr_1.dfr_resolve(node[field](...args));
    }
    static newAsync(mix, parent) {
        let query = new JsdomQuery(mix);
        query.ctx.owner = parent;
        query.then = query.ctx.thener;
        return query;
    }
    //#region driver utils
    unlock() {
    }
    //#endregion driver utils
    static build(config, setts) {
        throw new Error('No build for JSDom is required. Use direkt load');
    }
    static load(url, config, setts) {
        let query = new JsdomQuery();
        NetworkDriver_1.NetworkDriver.load(url, setts.opts).then(content => {
            let jsdom = new jsdom_1.JSDOM(content.toString());
            query.add(jsdom.window.document);
            query.resolve(query);
        });
        return query;
    }
    static fetch(url, config, setts) {
        return this.load(url, config, setts);
    }
    static setDriver(driver) {
        throw new Error('JSDOM does not support driver');
    }
    static getDriver(config, setts) {
        throw new Error('JSDOM does not support driver');
    }
    static unlockDriver(mix) {
    }
}
exports.JsdomQuery = JsdomQuery;
function toCamelCase(property) {
    return property.replace(/\-(\w)/g, (full, char) => {
        return char.toUpperCase();
    });
}
