"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const atma_utils_1 = require("atma-utils");
const SeleniumDriver_1 = require("./SeleniumDriver");
const driver_1 = require("./utils/driver");
const deco_1 = require("../utils/deco");
const CookieContainer_1 = require("../common/CookieContainer");
let POOL_DEFAULT = 5;
let POOL_CUSTOM;
class DriverPool {
    constructor() {
        this.pool = [];
        this.queue = [];
    }
    get(url = null, config, setts) {
        return __awaiter(this, void 0, void 0, function* () {
            if (setts) {
                let driver = DriverExtractor.extractDriver(setts.query);
                if (driver) {
                    if (this.singleton && this.singleton.driver === driver) {
                        this.singleton.busy = true;
                        return this.singleton;
                    }
                    let wrapper = this.pool.find(x => x.driver === driver);
                    if (wrapper == null) {
                        wrapper = new DriverWrapper();
                        wrapper.driver = driver;
                        wrapper.busy = true;
                        wrapper.requestedAt = new Date();
                        this.pool.push(wrapper);
                    }
                    return wrapper;
                }
                if (setts.pool) {
                    if (typeof setts.pool === 'number') {
                        POOL_CUSTOM = Math.max(setts.pool, this.pool.length);
                    }
                    return yield this.requestDriver(url, config);
                }
            }
            return yield this.getGlobal(url, config);
        });
    }
    getWithDomain(url = null, config, setts) {
        return __awaiter(this, void 0, void 0, function* () {
            let wrapper = yield this.get(url, config, setts);
            let match = /[^/]\/[^/]/.exec(url);
            let domain = match == null ? url : url.substring(0, match.index + 1);
            let currentUrl = yield wrapper.driver.getCurrentUrl();
            if (!currentUrl || !currentUrl.includes(domain.replace(/https?:\/\//, ''))) {
                yield wrapper.driver.get(domain);
            }
            return wrapper;
        });
    }
    unlockDriver(mix) {
        return __awaiter(this, void 0, void 0, function* () {
            let driver = DriverExtractor.extractDriver(mix);
            if (driver == null) {
                return;
            }
            let wrapper = this.pool.find(x => x.driver === driver);
            if (wrapper == null) {
                console.warn('SeleniumQuery. Unlock driver. Wrapper not found');
                return;
            }
            wrapper.busy = false;
            // Tick next awaiter
            let dfrData = this.queue.shift();
            if (dfrData) {
                wrapper.busy = true;
                yield wrapper.ensureCookies(dfrData.url, dfrData.config);
                dfrData.dfr.resolve(wrapper);
            }
        });
    }
    getGlobal(url = null, config) {
        return __awaiter(this, void 0, void 0, function* () {
            this.memCookies(url, config);
            let singleton = new DriverWrapper();
            yield singleton.build(config);
            yield singleton.ensureCookies(url, config);
            return (this.singleton = singleton);
        });
    }
    extractDriver(query) {
        return DriverExtractor.extractDriver(query);
    }
    requestDriver(url = null, config) {
        return __awaiter(this, void 0, void 0, function* () {
            this.memCookies(url, config);
            let free = this.pool.find(x => x.busy !== true);
            if (free) {
                free.busy = true;
                yield free.ensureCookies(url, config);
                return free;
            }
            if (this.pool.length < getPoolCount()) {
                let wrapper = new DriverWrapper();
                wrapper.busy = true;
                wrapper.requestedAt = new Date;
                this.pool.push(wrapper);
                yield wrapper.build(config);
                yield wrapper.ensureCookies(url, config);
                return wrapper;
            }
            let dfr = new atma_utils_1.class_Dfr();
            this.queue.push({ url, config, dfr });
            return dfr;
        });
    }
    memCookies(url, config) {
        if (config && config.cookies) {
            CookieContainer_1.cookieContainer.addCookies(url, config.cookies);
        }
    }
    setGlobal(driver) {
        this.singleton = new DriverWrapper();
        this.singleton.busy = false;
        this.singleton.driver = driver;
    }
}
__decorate([
    deco_1.singleton
], DriverPool.prototype, "getGlobal", null);
exports.DriverPool = DriverPool;
class DriverWrapper {
    constructor() {
        this.busy = false;
    }
    build(config) {
        return __awaiter(this, void 0, void 0, function* () {
            this.driver = yield SeleniumDriver_1.buildDriver(config);
        });
    }
    ensureCookies(url, config) {
        return __awaiter(this, void 0, void 0, function* () {
            let cookies = CookieContainer_1.cookieContainer.getCookies(url);
            if (!cookies || cookies === this.cookies) {
                return;
            }
            this.cookies = cookies;
            yield driver_1.ensureCookies(this.driver, url, cookies, config);
        });
    }
}
exports.DriverWrapper = DriverWrapper;
function getPoolCount() {
    return POOL_CUSTOM == null ? POOL_DEFAULT : POOL_CUSTOM;
}
exports.driverPool = new DriverPool();
var DriverExtractor;
(function (DriverExtractor) {
    function isElement(mix) {
        return mix != null && 'getDriver' in mix;
    }
    function isDriver(mix) {
        return mix != null && 'get' in mix && 'manage' in mix;
    }
    function fromQuery(mix) {
        let el = mix[0];
        if (isDriver(el)) {
            // is driver itself
            return el;
        }
        if (isElement(el)) {
            return el.getDriver();
        }
        return null;
    }
    function fromOwner(mix) {
        let owner = mix.ctx && mix.ctx.owner;
        let stack = [];
        while (owner != null) {
            let driver = fromQuery(owner);
            if (driver) {
                return driver;
            }
            stack.push(owner);
            owner = owner.ctx && owner.ctx.owner;
            if (stack.indexOf(owner) !== -1) {
                return null;
            }
        }
    }
    function fromWrapper(mix) {
        if (isDriver(mix)) {
            return mix;
        }
        if ('driver' in mix && 'busy' in mix) {
            return mix.driver;
        }
        return null;
    }
    function extractDriver(mix) {
        if (mix == null) {
            return null;
        }
        var driver = fromQuery(mix);
        if (driver)
            return driver;
        var driver = fromOwner(mix);
        if (driver)
            return driver;
        var driver = fromWrapper(mix);
        if (driver)
            return driver;
        return null;
    }
    DriverExtractor.extractDriver = extractDriver;
})(DriverExtractor || (DriverExtractor = {}));
