"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const driver_1 = require("./utils/driver");
const DriverPool_1 = require("./DriverPool");
const atma_utils_1 = require("atma-utils");
const WebdriverQuery_1 = require("./WebdriverQuery");
const SelectorsEx_1 = require("../common/SelectorsEx");
exports.Webdriver = {
    fromHtml(html, config) {
        return exports.Webdriver.load(`data:text/html;charset=utf-8,${html}`, config);
    },
    build(config, setts) {
        return DriverPool_1.driverPool
            .get(null, config, setts)
            .then(wrapper => wrapper.driver);
    },
    load(url, config, setts) {
        if (url[0] === '/') {
            url = 'file://' + process.cwd() + url;
        }
        let query = WebdriverQuery_1.WebdriverQuery.newAsync();
        DriverPool_1.driverPool
            .get(url, config, setts)
            .then(wrapper => {
            driver_1.loadUrl(wrapper.driver, url, config).then(driver => {
                query.add(driver);
                query.resolve(query);
            });
        }, error => query.reject(error));
        return query;
    },
    unlockDriver(mix) {
        DriverPool_1.driverPool.unlockDriver(mix);
    },
    fetch(url, config, setts) {
        let dfr = new atma_utils_1.class_Dfr;
        DriverPool_1.driverPool
            .getWithDomain(url, config, setts)
            .then(wrapper => {
            wrapper
                .driver
                .executeAsyncScript(scripts_fetchAsync, url, setts && setts.opts && JSON.stringify(setts.opts) || null)
                .then((result) => {
                let isError = result && result.name === 'Error';
                if (isError) {
                    DriverPool_1.driverPool.unlockDriver(wrapper);
                    dfr.reject(new Error(result.message));
                    return;
                }
                if ('findElements' in result || (atma_utils_1.is_ArrayLike(result) && result.length !== 0 && 'findElements' in result[0])) {
                    // Consumer is responsible to unlock later the driver
                    dfr.resolve(new WebdriverQuery_1.WebdriverQuery(result));
                    return;
                }
                DriverPool_1.driverPool.unlockDriver(wrapper);
                dfr.resolve(result);
            }, error => {
                dfr.reject(error);
            });
        });
        return dfr;
    },
    pseudo: SelectorsEx_1.SelectorsEx.pseudoFns
};
