"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("./utils/node");
const global_1 = require("../global");
const IQuery_1 = require("../common/IQuery");
const Webdriver_1 = require("./Webdriver");
const DriverPool_1 = require("./DriverPool");
const JsdomDriver_1 = require("../jsdom/JsdomDriver");
const CheerioDriver_1 = require("../cheerio/CheerioDriver");
const NetworkDriver_1 = require("../fetch/NetworkDriver");
const driver_1 = require("./utils/driver");
const SeleniumDriver_1 = require("./SeleniumDriver");
const WebdriverEventsPoll_1 = require("./WebdriverEventsPoll");
const SelectorsEx_1 = require("../common/SelectorsEx");
class WebdriverQuery extends IQuery_1.IQuery {
    hasClassFn(node, name) {
        return node_1.node_eval(node, scripts_nodeClassHas, name);
    }
    addClassFn(node, name) {
        return node_1.node_eval(node, scripts_nodeClassAdd, name);
    }
    removeClassFn(node, name) {
        return node_1.node_eval(node, scripts_nodeClassRemove, name);
    }
    toggleClassFn(node, name) {
        return node_1.node_eval(node, scripts_nodeClassToggle, name);
    }
    textGetFn(node) {
        return this.getField(node, 'textContent');
    }
    textSetFn(node, text) {
        return this.setField(node, 'textContent', text);
    }
    htmlOuterGetFn(node) {
        let driver = node_1.node_getDriver(node);
        if (driver === node) {
            return driver.getPageSource();
        }
        return this.getField(node, 'outerHTML');
    }
    htmlGetFn(node) {
        let driver = node_1.node_getDriver(node);
        if (driver === node) {
            return driver.getPageSource();
        }
        return this.getField(node, 'innerHTML');
    }
    htmlSetFn(node, text) {
        return this.setField(node, 'innerHTML', text);
    }
    appendFn(node, html) {
        return this.callField(node, 'insertAdjacentHTML', 'beforeend', html);
    }
    prependFn(node, html) {
        return this.callField(node, 'insertAdjacentHTML', 'afterbegin', html);
    }
    beforeFn(node, html) {
        return this.callField(node, 'insertAdjacentHTML', 'beforebegin', html);
    }
    afterFn(node, html) {
        return this.callField(node, 'insertAdjacentHTML', 'afterend', html);
    }
    cssGet(node, prop) {
        return new Promise((resolve, reject) => {
            node.getCssValue(prop).then(resolve, reject);
        });
    }
    cssSet(node, css) {
        return node_1.node_eval(node, scripts_nodeCss, css);
    }
    heightGetFn(node) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getBoundingClientRect(node)).height;
        });
    }
    widthGetFn(node) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getBoundingClientRect(node)).width;
        });
    }
    innerHeightFn(node) {
        return this.getField(node, 'offsetHeight');
    }
    innerWidthFn(node) {
        return this.getField(node, 'offsetWidth');
    }
    getBoundingClientRect(node) {
        return node_1.node_eval(node, scripts_nodeFunctionCall, 'getBoundingClientRect');
    }
    getPosition(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let dfrTop = this.getField(node, 'offsetTop');
            let dfrLeft = this.getField(node, 'offsetLeft');
            let [top, left] = yield Promise.all([dfrTop, dfrLeft]);
            return { top, left };
        });
    }
    scrollTopGetFn(node) {
        return this.getField(node, 'scrollTop');
    }
    scrollTopSetFn(node, scroll) {
        return this.setField(node, 'scrollTop', scroll);
    }
    scrollLeftGetFn(node) {
        return this.getField(node, 'scrollLeft');
    }
    scrollLeftSetFn(node, scroll) {
        return this.setField(node, 'scrollLeft', scroll);
    }
    evalFn(node, mix, ...args) {
        return node_1.node_eval(node, mix, ...args);
    }
    //#region Events
    clickFn(node) {
        return node.click();
    }
    triggerFn(node, type, ...args) {
        return node_1.node_eval(node, scripts_nodeTrigger, type, ...args);
    }
    selectFn(node, ...args) {
        return this.getField(node, 'tagName').then(name => {
            var fn = name === 'SELECT' ?
                scripts_nodeSelectOption :
                scripts_nodeSelectTextRange;
            return node_1.node_eval(node, fn, ...args);
        });
    }
    focusFn(node) {
        return this.callField(node, 'focus');
    }
    blurFn(node) {
        return this.callField(node, 'blur');
    }
    sendKeysFn(node, mix) {
        return node.sendKeys(mix);
    }
    typeFn(node, str) {
        var arr = Events.toSequance(str), fn = Events.getSequenceFunction(arr);
        return fn(node);
    }
    pressFn(node, str) {
        var key = Events.toCombination(str);
        return node.sendKeys(key);
    }
    //#endregion
    //#region Manipulate
    removeFn(node) {
        return node_1.node_eval(node, scripts_nodeRemove);
    }
    //#endregion
    //#region Properties
    attrGetFn(node, prop) {
        return new Promise((resolve, reject) => {
            node.getAttribute(prop).then(resolve, reject);
        });
    }
    attrSetFn(node, attr) {
        return node_1.node_eval(node, scripts_nodeAttribute, attr);
    }
    valGetFn(node) {
        return this.getField(node, 'value');
    }
    valSetFn(node, value) {
        return this.setField(node, 'value', value);
    }
    dataGetFn(node, key) {
        return node_1.node_eval(node, scripts_nodeDataset, key);
    }
    dataSetFn(node, data) {
        return node_1.node_eval(node, scripts_nodeDataset, data);
    }
    propGetFn(node, key) {
        return node_1.node_eval(node, scripts_nodeProperty, key);
    }
    propSetFn(node, data) {
        return node_1.node_eval(node, scripts_nodeProperty, data);
    }
    //#endregion
    findFn(node, selector) {
        return new Promise((resolve, reject) => {
            node.findElements({ css: selector }).then(resolve, reject);
        });
    }
    matchesFn(node, selector) {
        return node_1.node_eval(node, scripts_nodeMatchesSelector, selector);
    }
    parentFn(node) {
        return node_1.node_eval(node, scripts_nodeParent);
    }
    closestFn(node, sel) {
        return node_1.node_eval(node, scripts_nodeClosest, sel);
    }
    childrenFn(node, sel) {
        return node_1.node_eval(node, scripts_nodeChildren, sel);
    }
    nextFn(node, sel) {
        return node_1.node_eval(node, scripts_nodeNext, sel);
    }
    getField(node, field) {
        return node_1.node_eval(node, scripts_nodeProperty, field);
    }
    setField(node, mix, val) {
        if (arguments.length === 2) {
            return node_1.node_eval(node, scripts_nodeProperty, mix);
        }
        if (arguments.length === 3) {
            return node_1.node_eval(node, scripts_nodeProperty, mix, val);
        }
        return null;
    }
    callField(node, field, ...args) {
        return node_1.node_eval(node, scripts_nodeFunctionCall, field, ...args);
    }
    _onFn(node, type, cb) {
        return WebdriverEventsPoll_1.WebdriverEventsPoll.addEventListener(node, type, cb);
    }
    _offFn(node, type, cb) {
        return WebdriverEventsPoll_1.WebdriverEventsPoll.removeEventListener(node, type, cb);
    }
    _onOnceFn(node, type, cb) {
        const fn = function (event) {
            WebdriverEventsPoll_1.WebdriverEventsPoll.removeEventListener(node, type, fn);
            cb(event);
        };
        return WebdriverEventsPoll_1.WebdriverEventsPoll.addEventListener(node, type, fn);
    }
    //#region driver utils
    manage() {
        let driver = DriverPool_1.driverPool.extractDriver(this);
        if (driver == null) {
            console.log(this);
            throw new Error(`Driver not found in set`);
        }
        return driver.manage();
    }
    waitForPageLoad() {
        return driver_1.waitForPageLoad(this);
    }
    waitForPageReady() {
        return driver_1.waitForPageLoad(this, 'interactive');
    }
    waitForElement(selector) {
        return driver_1.waitForElement(this, selector);
    }
    waitForResource(selector) {
        return driver_1.driver_evalAsync(this, scripts_waitForResourceCallback, selector);
    }
    unlock() {
        Webdriver_1.Webdriver.unlockDriver(this);
    }
    //#endregion driver utils
    static build(config, setts) {
        return Webdriver_1.Webdriver.build(config, setts);
    }
    static load(url, config = SeleniumDriver_1.DefaultConfig, setts) {
        switch (config.name.toLowerCase()) {
            case 'jsdom':
                return JsdomDriver_1.JsdomDriver.load(url, config, setts);
            case 'cheerio':
                return CheerioDriver_1.CheerioDriver.load(url, config, setts);
            default:
                return Webdriver_1.Webdriver.load(url, config, setts);
        }
    }
    static fetch(url, config = SeleniumDriver_1.DefaultConfig, setts) {
        switch (config.name.toLowerCase()) {
            case 'jsdom':
                return JsdomDriver_1.JsdomDriver.fetch(url, config, setts);
            case 'cheerio':
                return CheerioDriver_1.CheerioDriver.fetch(url, config, setts);
            default:
                return Webdriver_1.Webdriver.fetch(url, config, setts);
        }
    }
    static setDriver(driver) {
        DriverPool_1.driverPool.setGlobal(driver);
    }
    static getDriver(config, setts) {
        return DriverPool_1.driverPool.get('', config, setts);
    }
    static unlockDriver(mix) {
        Webdriver_1.Webdriver.unlockDriver(mix);
    }
    static newAsync(mix, parent) {
        let query = new WebdriverQuery(mix);
        query.ctx.owner = parent;
        query.then = query.ctx.thener;
        return query;
    }
}
exports.WebdriverQuery = WebdriverQuery;
WebdriverQuery.cheerio = CheerioDriver_1.CheerioDriver;
WebdriverQuery.jsdom = JsdomDriver_1.JsdomDriver;
WebdriverQuery.network = NetworkDriver_1.NetworkDriver;
WebdriverQuery.pseudo = SelectorsEx_1.SelectorsEx.pseudoFns;
var Events;
(function (Events) {
    const Key = global_1.refs.Key;
    const aliases = {
        'ctrl': 'control',
        'backspace': 'back_space',
        'esc': 'escape',
        'left': 'arrow_left',
        'right': 'arrow_right',
        'up': 'arrow_up',
        'down': 'arrow_down',
    };
    function toSequance(str) {
        var delimiter = '_%%%%%%_';
        str = str.replace(/\{([\w_]+)\}/g, function (full, name) {
            var key = (aliases[name] || name).toUpperCase();
            if (key in Key === false) {
                return full;
            }
            return delimiter + key + delimiter;
        });
        var parts = str.split(delimiter);
        return parts.map((str, i) => {
            if (i % 2 === 0) {
                return str;
            }
            return Key[str];
        });
    }
    Events.toSequance = toSequance;
    function toCombination(str) {
        var keys = str.split('+');
        keys.forEach((x, i) => {
            if (x === '') {
                keys[i] = '+';
                return;
            }
            if (isSpecial(x)) {
                keys[i] = getSpecial(x);
                return;
            }
        });
        if (keys.length === 0) {
            return keys[0];
        }
        return Key.chord(...keys);
    }
    Events.toCombination = toCombination;
    ;
    function getSpecial(name) {
        var key = (aliases[name] || name).toUpperCase();
        return Key[key];
    }
    function isSpecial(name) {
        var key = (aliases[name] || name).toUpperCase();
        return key in Key;
    }
    function getSequenceFunction(arr) {
        return function (node) {
            var dfrs = arr.map(str => node.sendKeys(str));
            return dfrs[dfrs.length - 1];
        };
    }
    Events.getSequenceFunction = getSequenceFunction;
})(Events || (Events = {}));
