"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dfr_1 = require("../../utils/dfr");
const async_1 = require("../../utils/async");
const DriverPool_1 = require("../DriverPool");
const WebdriverQuery_1 = require("../WebdriverQuery");
const atma_utils_1 = require("atma-utils");
const node_1 = require("./node");
function loadUrl(driver, url, config) {
    return driver
        .get(url)
        .then(() => driver, (error) => {
        return error;
        // if (error.code !== 100) {
        //     return error;
        // }
        // return BuildStatics
        //     .build(config)
        //     .then(driver => {
        //         return driver
        //             .get(url)
        //             .then(() => driver);
        //     });
    });
}
exports.loadUrl = loadUrl;
function ensureCookies(driver, url, cookies, config) {
    return dfr_1.dfr_run((resolve, reject) => {
        if (!cookies) {
            resolve();
            return;
        }
        let arr = cookies.split(';').map(x => x.trim()).map(single => {
            let parts = single.split('=').map(x => x.trim());
            return { name: parts[0], value: parts[1] };
        });
        ;
        let origin = config.cookieOrigin;
        if (origin == null) {
            origin = url;
        }
        loadUrl(driver, origin, config).then(driver => {
            let dfrs = arr.map(cookie => driver.manage().addCookie(cookie));
            async_1._when(dfrs, () => {
                resolve();
            });
        });
    });
}
exports.ensureCookies = ensureCookies;
function driver_evalAsync(el, mix, ...args) {
    let set = WebdriverQuery_1.WebdriverQuery.newAsync(void 0, el);
    let script = node_1.node_toScript(mix);
    let driver = DriverPool_1.driverPool.extractDriver(el);
    if (driver == null) {
        set.reject(new Error('Driver is not resolved.'));
        return set;
    }
    driver
        .executeAsyncScript(script, ...args)
        .then((result) => {
        set.resolve(Promise.resolve(result));
    }, error => {
        console.error('Unexpected browser error', error);
        set.reject(error);
    });
    return set;
}
exports.driver_evalAsync = driver_evalAsync;
function waitForElement(query, selector) {
    let driver = DriverPool_1.driverPool.extractDriver(query);
    let set = WebdriverQuery_1.WebdriverQuery.newAsync(void 0, query);
    if (driver == null) {
        set.reject(new Error(`Driver not found in set`));
        return;
    }
    waitForTrue(() => {
        return query.find(selector).then(x => {
            return x.length > 0;
        });
    }, 10000).then(() => {
        query.find(selector).then(x => set.resolve(x), err => set.reject(err));
    }, (err) => set.reject(err));
    return set;
}
exports.waitForElement = waitForElement;
function waitForPageLoad(query, waitForState = 'complete') {
    let driver = DriverPool_1.driverPool.extractDriver(query);
    let set = WebdriverQuery_1.WebdriverQuery.newAsync(null, query);
    if (driver == null) {
        set.reject(new Error(`Driver not found in set`));
        return set;
    }
    let delay = WaitForPageLoad.delay();
    let q = async_1.async_toThenable(query);
    async_1.async_all([q, delay]).then(([query]) => {
        let awaiters = [
            () => WaitForPageLoad.documentState(driver, 5000, waitForState),
        ];
        if (query.length > 0 && query[0] !== driver) {
            /* If element is passed, listen also for the element to be destroyed on page unload */
            let el = query[0];
            awaiters.unshift(() => WaitForPageLoad.elementLeavesDom(driver, el, 5000));
        }
        async_1.async_waterfallFn(...awaiters).then(() => {
            set.add(driver);
            set.resolve(set);
        }, error => set.reject(error));
    });
    return set;
}
exports.waitForPageLoad = waitForPageLoad;
var WaitForPageLoad;
(function (WaitForPageLoad) {
    function delay() {
        let dfr = new atma_utils_1.class_Dfr;
        setTimeout(() => dfr.resolve(), 100);
        return dfr;
    }
    WaitForPageLoad.delay = delay;
    function documentState(driver, timeout, waitForState = 'complete') {
        let dfr = new atma_utils_1.class_Dfr;
        waitForTrue(isReady, timeout).then(() => {
            dfr.resolve();
        }, error => {
            dfr.reject(new Error(`ReadyState timeout`));
        });
        return dfr;
        function isReady() {
            return driver
                .executeScript('return document.readyState')
                .then(state => {
                if (waitForState === 'interactive') {
                    return state === 'interactive' || state === 'complete';
                }
                return state === waitForState;
            });
        }
    }
    WaitForPageLoad.documentState = documentState;
    function elementLeavesDom(driver, el, timeout) {
        let dfr = new atma_utils_1.class_Dfr;
        waitForTrue(isStale, timeout).then(x => {
            dfr.resolve();
        }, error => {
            dfr.reject(new Error(`The old element is still in dom after ${timeout}ms. Reload is not triggered`));
        });
        return dfr;
        function isStale() {
            return dfr_1.dfr_run((resolve, reject) => {
                el.getTagName().then(x => {
                    resolve(false);
                }, x => {
                    // waiting for StaleElementReferenceError
                    resolve(true);
                });
            });
        }
    }
    WaitForPageLoad.elementLeavesDom = elementLeavesDom;
})(WaitForPageLoad || (WaitForPageLoad = {}));
function waitForTrue(check, timeout) {
    let dfr = new atma_utils_1.class_Dfr;
    let time = Date.now();
    function tick() {
        check().then(function (state) {
            if (state === true) {
                dfr.resolve();
                return;
            }
            if (Date.now() - time > timeout) {
                dfr.reject(new Error('Timeout error'));
                return;
            }
            setTimeout(tick, 400);
        }, error => dfr.reject(error));
    }
    tick();
    return dfr;
}
