"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dfr_1 = require("../../utils/dfr");
function node_eval(node, mix, ...args) {
    return dfr_1.dfr_run((resolve, reject) => {
        var script = node_toScript(mix);
        var driver = node_getDriver(node);
        if (driver == null) {
            reject(new Error('Driver is not resolved.'));
            return;
        }
        let arr = node == driver ? args : [node, ...args];
        driver
            .executeScript(script, ...arr)
            .then(result => {
            resolve(result);
        }, error => {
            console.error('Unexpected sync browser error', error, 'for', script);
            resolve();
        });
    });
}
exports.node_eval = node_eval;
;
function node_evalAsync(node, mix, ...args) {
    return dfr_1.dfr_run((resolve, reject) => {
        var script = node_toScript(mix);
        var driver = node_getDriver(node);
        if (driver == null) {
            reject(new Error('Driver is not resolved.'));
            return;
        }
        driver
            .executeAsyncScript(script, node, ...args)
            .then(resolve, error => {
            console.error('Unexpected async browser error', error, 'for', script);
            resolve();
        });
    });
}
exports.node_evalAsync = node_evalAsync;
;
function node_getDriver(node) {
    if ('executeScript' in node) {
        return node;
    }
    if ('getDriver' in node) {
        return node.getDriver();
    }
    return node.driver_;
}
exports.node_getDriver = node_getDriver;
function node_toScript(mix) {
    if (typeof mix === 'string') {
        return mix;
    }
    var script = mix.toString();
    var arrowRgx = /^[^\)]+\)\s*=>/;
    if (arrowRgx.test(script)) {
        script = script.replace(arrowRgx, '').trim();
        if (script[0] !== '{') {
            // oneliner
            return `return ${script}`;
        }
    }
    script = script.substring(script.indexOf('{') + 1);
    script = script.substring(0, script.lastIndexOf('}') - 1);
    script = script.trim();
    return script;
}
exports.node_toScript = node_toScript;
